/******************************************************************************
 *
 * capabilities.c
 *
 * /proc/xen/capabilities
 *
 * Copyright (c) 2002-2004, K A Fraser, B Dragovic
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation; or, when distributed
 * separately from the Linux kernel or incorporated into other
 * software packages, subject to the following license:
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this source file (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <linux/proc_fs.h>
#include <linux/module.h>
#include <asm/xen/hypervisor.h>

static int capabilities_read(char *page, char **start, off_t off,
			     int count, int *eof, void *data)
{
	int len = 0;
	*page = 0;

	if (is_initial_xendomain())
		len = sprintf(page, "control_d\n");

	*eof = 1;
	return len;
}

int __init capabilities_create_proc_entry(void)
{
	struct proc_dir_entry *entry;

	entry = create_proc_entry("xen/capabilities", 0400, NULL);
	if (!entry)
		return -ENOMEM;

	entry->owner = THIS_MODULE;
	entry->read_proc = capabilities_read;

	return 0;
}

void __exit capabilities_remove_proc_entry(void)
{
	remove_proc_entry("xen/capabilities", NULL);
}
