/******************************************************************************
 *
 * main.c
 *
 * Xen userspace control interfaces
 *
 * Copyright (c) 2002-2004, K A Fraser, B Dragovic
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation; or, when distributed
 * separately from the Linux kernel or incorporated into other
 * software packages, subject to the following license:
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this source file (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include "xenctrl.h"

#include <linux/proc_fs.h>
#include <linux/module.h>
#include <asm/xen/hypervisor.h>

static int __init xenctrl_init(void)
{
	struct proc_dir_entry *dir;
	int ret;

	if (!is_running_on_xen())
		return -ENODEV;

	dir = proc_mkdir("xen", NULL);
	if (!dir)
		return -ENOMEM;

	dir->owner = THIS_MODULE;

	ret = capabilities_create_proc_entry();
	if (ret)
		goto fail1;

	ret = privcmd_create_proc_entry();
	if (ret)
		goto fail2;

	ret = xenbus_create_proc_entry();
	if (ret)
		goto fail3;

	ret = sys_hypervisor_init();
	if (ret)
		goto fail4;

	return 0;

 fail4: xenbus_remove_proc_entry();
 fail3: privcmd_remove_proc_entry();
 fail2: capabilities_remove_proc_entry();
 fail1:	remove_proc_entry("xen", NULL);
	return ret;
}

static void __exit xenctrl_exit(void)
{
	sys_hypervisor_exit();
	xenbus_remove_proc_entry();
	privcmd_remove_proc_entry();
	capabilities_remove_proc_entry();
	remove_proc_entry("xen", NULL);
}

module_init(xenctrl_init);
module_exit(xenctrl_exit);

MODULE_LICENSE("Dual BSD/GPL");
